# -*- coding: utf-8 -*-
import os
import re
import cgi
import subprocess
from os import path

from docutils import nodes

from sphinx import addnodes
from sphinx.builders.html import StandaloneHTMLBuilder

_idpattern = re.compile(
    r'(?P<title>.+) (\((?P<id>[\w\.]+)( (?P<descr>\w+))?\))$')


# Qt Help Collection Project (.qhcp).
# Is the input file for the help collection generator.
# It contains references to compressed help files which should be
# included in the collection.
# It may contain various other information for customizing Qt Assistant.
collection_template = '''\
<?xml version="1.0" encoding="utf-8" ?>
<QHelpCollectionProject version="1.0">
    <docFiles>
        <generate>
            <file>
                <input>%(outname)s.qhp</input>
                <output>%(outname)s.qch</output>
            </file>
        </generate>
        <register>
            <file>%(outname)s.qch</file>
        </register>
    </docFiles>
    <assistant>
         <title>calibre User Manual</title>
         <applicationIcon>_static/logo.png</applicationIcon>
         <enableDocumentationManager>false</enableDocumentationManager>
         <enableAddressBar visible="false">true</enableAddressBar>
         <cacheDirectory>calibre/user_manual</cacheDirectory>
         <aboutMenuText>
             <text>About calibre</text>
         </aboutMenuText>
         <aboutDialog>
             <file>about.txt</file>
             <icon>_static/logo.png</icon>
         </aboutDialog>
     </assistant>
</QHelpCollectionProject>
'''

ABOUT='''\
calibre is the one stop solution for all your ebook needs. It was created
originally by Kovid Goyal, to help him manage his ebook collection and is now
very actively developed by an international community of ebook enthusiasts.

Its goal is to empower you, the user, to do whatever you like with the ebooks
in your collection. You can convert them to many different formats, read them
on your computer, send them to many different devices, edit their metadata
and covers, etc.

calibre also allows you to download news from a variety of different sources all
over the Internet and read conveniently in ebooks form. In keeping with its
philosophy of empowering the user, it has a simple system to allow you to add
your own favorite news sources. In fact, most the built-in news sources in
calibre were originally contributed by users.
'''

# Qt Help Project (.qhp)
# This is the input file for the help generator.
# It contains the table of contents, indices and references to the
# actual documentation files (*.html).
# In addition it defines a unique namespace for the documentation.
project_template = '''\
<?xml version="1.0" encoding="UTF-8"?>
<QtHelpProject version="1.0">
    <namespace>%(outname)s.org.%(outname)s.%(nversion)s</namespace>
    <virtualFolder>doc</virtualFolder>
    <customFilter name="%(project)s %(version)s">
        <filterAttribute>%(outname)s</filterAttribute>
        <filterAttribute>%(version)s</filterAttribute>
    </customFilter>
    <filterSection>
        <filterAttribute>%(outname)s</filterAttribute>
        <filterAttribute>%(version)s</filterAttribute>
        <toc>
            <section title="%(title)s" ref="%(masterdoc)s.html">
%(sections)s
            </section>
        </toc>
        <files>
%(files)s
        </files>
    </filterSection>
</QtHelpProject>
'''

section_template = '<section title="%(title)s" ref="%(ref)s"/>'
file_template = ' '*12 + '<file>%(filename)s</file>'


class QtHelpBuilder(StandaloneHTMLBuilder):
    """
    Builder that also outputs Qt help project, contents and index files.
    """
    name = 'qthelp'

    # don't copy the reST source
    copysource = False
    supported_image_types = ['image/svg+xml', 'image/png', 'image/gif',
                             'image/jpeg']

    # don't add links
    add_permalinks = False
    # don't add sidebar etc.
    embedded = True

    def init(self):
        StandaloneHTMLBuilder.init(self)
        # the output files for HTML help must be .html only
        self.out_suffix = '.html'
        self.link_suffix = '.html'
        #self.config.html_style = 'traditional.css'

    def handle_finish(self):
        self.build_qhcp(self.outdir, self.config.qthelp_basename)
        self.build_qhp(self.outdir, self.config.qthelp_basename)
        self.build_qhc(self.outdir, self.config.qthelp_basename)

    def build_qhc(self, outdir, outname):
        self.info('create Qt Help Collection...')
        with open(os.path.join(outdir, 'about.txt'), 'wb') as f:
            f.write(ABOUT)
        qhcp = os.path.abspath(os.path.join(outdir, outname+'.qhcp'))
        subprocess.check_call(['qcollectiongenerator', qhcp])
        qhc = qhcp[:-5]+'.qhc'
        self.info('Qt Help Collection: '+qhc)
        self.info('To test: assistant -collectionFile '+qhc)

    def build_qhcp(self, outdir, outname):
        self.info('writing collection project file...')
        f = open(path.join(outdir, outname+'.qhcp'), 'w')
        try:
            f.write(collection_template % {'outname': outname})
        finally:
            f.close()

    def build_qhp(self, outdir, outname):
        self.info('writing project file...')

        # sections
        tocdoc = self.env.get_and_resolve_doctree(self.config.master_doc, self,
                                                  prune_toctrees=False)
        istoctree = lambda node: (
                        isinstance(node, addnodes.compact_paragraph)
                            and node.has_key('toctree'))
        sections = []
        for node in tocdoc.traverse(istoctree):
            sections.extend(self.write_toc(node))

        if self.config.html_use_modindex:
            item = section_template % {'title': 'Global Module Index',
                                       'ref': 'modindex.html'}
            sections.append(' '*4*4 + item)
        sections = '\n'.join(sections)

        # keywords
        keywords = []
        index = self.env.create_index(self)
        for (key, group) in index:
            for title, (refs, subitems) in group:
                keywords.extend(self.build_keywords(title, refs, subitems))
        keywords = '\n'.join(keywords)

        # files
        if not outdir.endswith(os.sep):
            outdir += os.sep
        olen = len(outdir)
        projectfiles = []
        staticdir = path.join(outdir, '_static')
        imagesdir = path.join(outdir, '_images')
        for root, dirs, files in os.walk(outdir):
            resourcedir = root.startswith(staticdir) or root.startswith(imagesdir)
            for fn in files:
                if (resourcedir and not fn.endswith('.js')) or \
                       fn.endswith('.html'):
                    filename = path.join(root, fn)[olen:]
                    #filename = filename.replace(os.sep, '\\') # XXX
                    projectfiles.append(file_template % {'filename': filename})
        projectfiles = '\n'.join(projectfiles)

        # write the project file
        f = open(path.join(outdir, outname+'.qhp'), 'w')
        try:
            nversion = self.config.version.replace('.', '_')
            nversion = nversion.replace(' ', '_')
            f.write(project_template % {'outname': outname,
                                        'title': self.config.html_title,
                                        'version': self.config.version,
                                        'project': self.config.project,
                                        'nversion': nversion,
                                        'masterdoc': self.config.master_doc,
                                        'sections': sections,
                                        'keywords': keywords,
                                        'files': projectfiles})
        finally:
            f.close()

    def isdocnode(self, node):
        if not isinstance(node, nodes.list_item):
            return False
        if len(node.children) != 2:
            return False
        if not isinstance(node.children[0], addnodes.compact_paragraph):
            return False
        if not isinstance(node.children[0][0], nodes.reference):
            return False
        if not isinstance(node.children[1], nodes.bullet_list):
            return False
        return True

    def write_toc(self, node, indentlevel=4):
        parts = []
        if self.isdocnode(node):
            refnode = node.children[0][0]
            link = refnode['refuri']
            title = cgi.escape(refnode.astext()).replace('"','&quot;')
            item = '<section title="%(title)s" ref="%(ref)s">' % {
                                                                'title': title,
                                                                'ref': link}
            parts.append(' '*4*indentlevel + item)
            for subnode in node.children[1]:
                parts.extend(self.write_toc(subnode, indentlevel+1))
            parts.append(' '*4*indentlevel + '</section>')
        elif isinstance(node, nodes.list_item):
            for subnode in node:
                parts.extend(self.write_toc(subnode, indentlevel))
        elif isinstance(node, nodes.reference):
            link = node['refuri']
            title = cgi.escape(node.astext()).replace('"','&quot;')
            item = section_template % {'title': title, 'ref': link}
            item = ' '*4*indentlevel + item.encode('ascii', 'xmlcharrefreplace')
            parts.append(item.encode('ascii', 'xmlcharrefreplace'))
        elif isinstance(node, nodes.bullet_list):
            for subnode in node:
                parts.extend(self.write_toc(subnode, indentlevel))
        elif isinstance(node, addnodes.compact_paragraph):
            for subnode in node:
                parts.extend(self.write_toc(subnode, indentlevel))

        return parts

    def keyword_item(self, name, ref):
        matchobj = _idpattern.match(name)
        if matchobj:
            groupdict = matchobj.groupdict()
            shortname = groupdict['title']
            id = groupdict.get('id')
#            descr = groupdict.get('descr')
            if shortname.endswith('()'):
                shortname = shortname[:-2]
            id = '%s.%s' % (id, shortname)
        else:
            id = None

        if id:
            item = ' '*12 + '<keyword name="%s" id="%s" ref="%s"/>' % (
                                                                name, id, ref)
        else:
            item = ' '*12 + '<keyword name="%s" ref="%s"/>' % (name, ref)
        item.encode('ascii', 'xmlcharrefreplace')
        return item

    def build_keywords(self, title, refs, subitems):
        keywords = []

        title = cgi.escape(title)
#        if len(refs) == 0: # XXX
#            write_param('See Also', title)
        if len(refs) == 1:
            keywords.append(self.keyword_item(title, refs[0]))
        elif len(refs) > 1:
            for i, ref in enumerate(refs):  # XXX
#                item = (' '*12 +
#                        '<keyword name="%s [%d]" ref="%s"/>' % (
#                                                        title, i, ref))
#                item.encode('ascii', 'xmlcharrefreplace')
#                keywords.append(item)
                keywords.append(self.keyword_item(title, ref))

        if subitems:
            for subitem in subitems:
                keywords.extend(self.build_keywords(subitem[0], subitem[1], []))

        return keywords



#!/usr/bin/env python
# vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
from __future__ import (unicode_literals, division, absolute_import,
                        print_function)

__license__   = 'GPL v3'
__copyright__ = '2012, Kovid Goyal <kovid@kovidgoyal.net>'
__docformat__ = 'restructuredtext en'

import sys, os, shlex, subprocess, shutil, unicodedata

from calibre import prints, as_unicode, walk
from calibre.constants import iswindows, __appname__
from calibre.ptempfile import TemporaryDirectory, TemporaryFile
from calibre.libunzip import extract as zipextract
from calibre.utils.zipfile import ZipFile, ZIP_DEFLATED, ZIP_STORED
from calibre.utils.ipc.simple_worker import WorkerError

class Error(ValueError):
    pass

def ask_cli_question(msg):
    prints(msg, end=' [y/N]: ')
    sys.stdout.flush()

    if iswindows:
        import msvcrt
        ans = msvcrt.getch()
    else:
        import tty, termios
        old_settings = termios.tcgetattr(sys.stdin.fileno())
        try:
            tty.setraw(sys.stdin.fileno())
            try:
                ans = sys.stdin.read(1)
            except KeyboardInterrupt:
                ans = b''
        finally:
            termios.tcsetattr(sys.stdin.fileno(), termios.TCSADRAIN, old_settings)
    print()
    return ans == b'y'

def mobi_exploder(path, tdir, question=lambda x:True):
    from calibre.ebooks.mobi.tweak import explode, BadFormat
    try:
        return explode(path, tdir, question=question)
    except BadFormat as e:
        raise Error(as_unicode(e))

def zip_exploder(path, tdir, question=lambda x:True):
    zipextract(path, tdir)
    for f in walk(tdir):
        if f.lower().endswith('.opf'):
            return f
    raise Error('Invalid book: Could not find .opf')

def zip_rebuilder(tdir, path):
    with ZipFile(path, 'w', compression=ZIP_DEFLATED) as zf:
        # Write mimetype
        mt = os.path.join(tdir, 'mimetype')
        if os.path.exists(mt):
            zf.write(mt, 'mimetype', compress_type=ZIP_STORED)
        # Write everything else
        exclude_files = {'.DS_Store', 'mimetype', 'iTunesMetadata.plist'}
        for root, dirs, files in os.walk(tdir):
            for fn in files:
                if fn in exclude_files:
                    continue
                absfn = os.path.join(root, fn)
                zfn = unicodedata.normalize('NFC', os.path.relpath(absfn, tdir).replace(os.sep, '/'))
                zf.write(absfn, zfn)

def get_tools(fmt):
    fmt = fmt.lower()

    if fmt in {'mobi', 'azw', 'azw3'}:
        from calibre.ebooks.mobi.tweak import rebuild
        ans = mobi_exploder, rebuild
    elif fmt in {'epub', 'htmlz'}:
        ans = zip_exploder, zip_rebuilder
    else:
        ans = None, None

    return ans

def tweak(ebook_file):
    ''' Command line interface to the Tweak Book tool '''
    fmt = ebook_file.rpartition('.')[-1].lower()
    exploder, rebuilder = get_tools(fmt)
    if exploder is None:
        prints('Cannot tweak %s files. Supported formats are: EPUB, HTMLZ, AZW3, MOBI'
                , file=sys.stderr)
        raise SystemExit(1)

    with TemporaryDirectory('_tweak_'+
            os.path.basename(ebook_file).rpartition('.')[0]) as tdir:
        try:
            opf = exploder(ebook_file, tdir, question=ask_cli_question)
        except WorkerError as e:
            prints('Failed to unpack', ebook_file)
            prints(e.orig_tb)
            raise SystemExit(1)
        except Error as e:
            prints(as_unicode(e), file=sys.stderr)
            raise SystemExit(1)

        if opf is None:
            # The question was answered with No
            return

        ed = os.environ.get('EDITOR', 'dummy')
        cmd = shlex.split(ed)
        isvim = bool([x for x in cmd[0].split('/') if x.endswith('vim')])

        proceed = False
        prints('Book extracted to', tdir)

        if not isvim:
            prints('Make your tweaks and once you are done,', __appname__,
                    'will rebuild', ebook_file, 'from', tdir)
            print()
            proceed = ask_cli_question('Rebuild ' + ebook_file + '?')
        else:
            base = os.path.basename(ebook_file)
            with TemporaryFile(base+'.zip') as zipf:
                with ZipFile(zipf, 'w') as zf:
                    zf.add_dir(tdir)
                try:
                    subprocess.check_call(cmd + [zipf])
                except:
                    prints(ed, 'failed, aborting...')
                    raise SystemExit(1)
                with ZipFile(zipf, 'r') as zf:
                    shutil.rmtree(tdir)
                    os.mkdir(tdir)
                    zf.extractall(path=tdir)
            proceed = True

        if proceed:
            prints('Rebuilding', ebook_file, 'please wait ...')
            try:
                rebuilder(tdir, ebook_file)
            except WorkerError as e:
                prints('Failed to rebuild', ebook_file)
                prints(e.orig_tb)
                raise SystemExit(1)
            prints(ebook_file, 'successfully tweaked')




#!/usr/bin/env python
# vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
from __future__ import with_statement

__license__   = 'GPL v3'
__copyright__ = '2009, Kovid Goyal <kovid@kovidgoyal.net>'
__docformat__ = 'restructuredtext en'

import os, shutil, subprocess, sys

from setup import __appname__, __version__, basenames
from setup.build_environment import is64bit

WIXP = r'C:\Program Files (x86)\WiX Toolset v3.8'
if is64bit:
    UPGRADE_CODE = '5DD881FF-756B-4097-9D82-8C0F11D521EA'
else:
    UPGRADE_CODE = 'BEB2A80D-E902-4DAD-ADF9-8BD2DA42CFE1'
MINVERHUMAN = 'Windows Vista'

CANDLE = WIXP+r'\bin\candle.exe'
LIGHT = WIXP+r'\bin\light.exe'

class WixMixIn:

    def create_installer(self):
        self.installer_dir = self.j(self.src_root, 'build', 'wix')
        if os.path.exists(self.installer_dir):
            shutil.rmtree(self.installer_dir)
        os.makedirs(self.installer_dir)

        template = open(self.j(self.d(__file__), 'wix-template.xml'),
                'rb').read()

        components = self.get_components_from_files()
        wxs = template.format(
            app                = __appname__,
            appfolder          = 'Calibre2' if is64bit else __appname__,
            version            = __version__,
            upgrade_code       = UPGRADE_CODE,
            ProgramFilesFolder = 'ProgramFiles64Folder' if is64bit else 'ProgramFilesFolder',
            x64                = ' 64bit' if is64bit else '',
            minverhuman        = MINVERHUMAN,
            minver             = '600',
            fix_wix = '<Custom Action="OverwriteWixSetDefaultPerMachineFolder" After="WixSetDefaultPerMachineFolder" />' if is64bit else '',
            compression        = self.opts.msi_compression,
            app_components     = components,
            exe_map            = self.smap,
            main_icon          = self.j(self.src_root, 'icons', 'library.ico'),
            web_icon           = self.j(self.src_root, 'icons', 'web.ico'),
        )
        template = open(self.j(self.d(__file__), 'en-us.xml'),
                'rb').read()
        enus = template.format(app=__appname__)


        enusf = self.j(self.installer_dir, 'en-us.wxl')
        wxsf = self.j(self.installer_dir, __appname__+'.wxs')
        with open(wxsf, 'wb') as f:
            f.write(wxs)
        with open(enusf, 'wb') as f:
            f.write(enus)
        wixobj = self.j(self.installer_dir, __appname__+'.wixobj')
        arch = 'x64' if is64bit else 'x86'
        cmd = [CANDLE, '-nologo', '-arch', arch, '-ext', 'WiXUtilExtension', '-o', wixobj, wxsf]
        self.info(*cmd)
        self.run_wix(cmd)
        self.installer = self.j(self.src_root, 'dist')
        if not os.path.exists(self.installer):
            os.makedirs(self.installer)
        self.installer = self.j(self.installer, '%s%s-%s.msi' % (__appname__,
            ('-64bit' if is64bit else ''), __version__))
        license = self.j(self.src_root, 'LICENSE.rtf')
        banner  = self.j(self.src_root, 'icons', 'wix-banner.bmp')
        dialog  = self.j(self.src_root, 'icons', 'wix-dialog.bmp')
        cmd = [LIGHT, '-nologo', '-ext', 'WixUIExtension',
                '-cultures:en-us', '-loc', enusf, wixobj,
                '-ext', 'WixUtilExtension',
                '-o', self.installer,
                '-dWixUILicenseRtf='+license,
                '-dWixUIBannerBmp='+banner,
                '-dWixUIDialogBmp='+dialog]
        cmd.extend([
            '-sice:ICE60',# No language in dlls warning
            '-sice:ICE61',# Allow upgrading with same version number
            '-sice:ICE40', # Re-install mode overriden
            '-sice:ICE69', # Shortcut components are part of a different feature than the files they point to
        ])
        if self.opts.no_ice:
            cmd.append('-sval')
        if self.opts.verbose:
            cmd.append('-v')
        self.info(*cmd)
        self.run_wix(cmd)

    def run_wix(self, cmd):
        p = subprocess.Popen(cmd, stdout=subprocess.PIPE,
                              stderr=subprocess.PIPE)
        ret = p.wait()
        self.info(p.stdout.read())
        self.info(p.stderr.read())
        if ret != 0:
            sys.exit(1)

    def get_components_from_files(self):

        self._file_idc = 0
        self.file_id_map = {}

        def process_dir(path):
            components = []
            for x in os.listdir(path):
                self._file_idc += 1
                f = os.path.join(path, x)
                self.file_id_map[f] = fid = self._file_idc

                if os.path.isdir(f):
                    components.append(
                        '<Directory Id="file_%s" FileSource="%s" Name="%s">' %
                        (self.file_id_map[f], f, x))
                    c = process_dir(f)
                    components.extend(c)
                    components.append('</Directory>')
                else:
                    checksum = 'Checksum="yes"' if x.endswith('.exe') else ''
                    c = [
                    ('<Component Id="component_%s" Feature="MainApplication" '
                     'Guid="*">') % (fid,),
                    ('<File Id="file_%s" Source="%s" Name="%s" ReadOnly="yes" '
                     'KeyPath="yes" %s/>') %
                        (fid, f, x, checksum),
                    '</Component>'
                    ]
                    if x.endswith('.exe') and not x.startswith('pdf'):
                        # Add the executable to app paths so that users can
                        # launch it from the run dialog even if it is not on
                        # the path. See http://msdn.microsoft.com/en-us/library/windows/desktop/ee872121(v=vs.85).aspx
                        c[-1:-1] = [
                        ('<RegistryValue Root="HKLM" '
                         r'Key="SOFTWARE\Microsoft\Windows\CurrentVersion\App '
                         r'Paths\%s" Value="[#file_%d]" Type="string" />'%(x, fid)),
                        ('<RegistryValue Root="HKLM" '
                         r'Key="SOFTWARE\Microsoft\Windows\CurrentVersion\App '
                         r'Paths\{0}" Name="Path" Value="[APPLICATIONFOLDER]" '
                         'Type="string" />'.format(x)),
                        ]
                    components.append('\n'.join(c))
            return components

        components = process_dir(os.path.abspath(self.base))
        self.smap = {}
        for x in basenames['gui']:
            self.smap[x] = 'file_%d'%self.file_id_map[self.a(self.j(self.base, x+'.exe'))]

        return '\t\t\t\t'+'\n\t\t\t\t'.join(components)





version = 1.1
path = '/Applications/iTunes.app'

classes = \
[('print_settings', 'pset'),
 ('application', 'capp'),
 ('artwork', 'cArt'),
 ('audio_CD_playlist', 'cCDP'),
 ('audio_CD_track', 'cCDT'),
 ('browser_window', 'cBrW'),
 ('device_playlist', 'cDvP'),
 ('device_track', 'cDvT'),
 ('encoder', 'cEnc'),
 ('EQ_preset', 'cEQP'),
 ('EQ_window', 'cEQW'),
 ('file_track', 'cFlT'),
 ('folder_playlist', 'cFoP'),
 ('item', 'cobj'),
 ('library_playlist', 'cLiP'),
 ('playlist', 'cPly'),
 ('playlist_window', 'cPlW'),
 ('radio_tuner_playlist', 'cRTP'),
 ('shared_track', 'cShT'),
 ('source', 'cSrc'),
 ('track', 'cTrk'),
 ('URL_track', 'cURT'),
 ('user_playlist', 'cUsP'),
 ('visual', 'cVis'),
 ('window', 'cwin')]

enums = \
[('track_listing', 'kTrk'),
 ('album_listing', 'kAlb'),
 ('cd_insert', 'kCDi'),
 ('standard', 'lwst'),
 ('detailed', 'lwdt'),
 ('stopped', 'kPSS'),
 ('playing', 'kPSP'),
 ('paused', 'kPSp'),
 ('fast_forwarding', 'kPSF'),
 ('rewinding', 'kPSR'),
 ('off', 'kRpO'),
 ('one', 'kRp1'),
 ('all', 'kAll'),
 ('small', 'kVSS'),
 ('medium', 'kVSM'),
 ('large', 'kVSL'),
 ('library', 'kLib'),
 ('iPod', 'kPod'),
 ('audio_CD', 'kACD'),
 ('MP3_CD', 'kMCD'),
 ('device', 'kDev'),
 ('radio_tuner', 'kTun'),
 ('shared_library', 'kShd'),
 ('unknown', 'kUnk'),
 ('albums', 'kSrL'),
 ('artists', 'kSrR'),
 ('composers', 'kSrC'),
 ('displayed', 'kSrV'),
 ('songs', 'kSrS'),
 ('none', 'kNon'),
 ('Books', 'kSpA'),
 ('folder', 'kSpF'),
 ('Genius', 'kSpG'),
 ('iTunes_U', 'kSpU'),
 ('Library', 'kSpL'),
 ('Movies', 'kSpI'),
 ('Music', 'kSpZ'),
 ('Party_Shuffle', 'kSpS'),
 ('Podcasts', 'kSpP'),
 ('Purchased_Music', 'kSpM'),
 ('TV_Shows', 'kSpT'),
 ('movie', 'kVdM'),
 ('music_video', 'kVdV'),
 ('TV_show', 'kVdT'),
 ('user', 'kRtU'),
 ('computed', 'kRtC')]

properties = \
[('copies', 'lwcp'),
 ('collating', 'lwcl'),
 ('starting_page', 'lwfp'),
 ('ending_page', 'lwlp'),
 ('pages_across', 'lwla'),
 ('pages_down', 'lwld'),
 ('error_handling', 'lweh'),
 ('requested_print_time', 'lwqt'),
 ('printer_features', 'lwpf'),
 ('fax_number', 'faxn'),
 ('target_printer', 'trpr'),
 ('current_encoder', 'pEnc'),
 ('current_EQ_preset', 'pEQP'),
 ('current_playlist', 'pPla'),
 ('current_stream_title', 'pStT'),
 ('current_stream_URL', 'pStU'),
 ('current_track', 'pTrk'),
 ('current_visual', 'pVis'),
 ('EQ_enabled', 'pEQ '),
 ('fixed_indexing', 'pFix'),
 ('frontmost', 'pisf'),
 ('full_screen', 'pFSc'),
 ('name', 'pnam'),
 ('mute', 'pMut'),
 ('player_position', 'pPos'),
 ('player_state', 'pPlS'),
 ('selection', 'sele'),
 ('sound_volume', 'pVol'),
 ('version', 'vers'),
 ('visuals_enabled', 'pVsE'),
 ('visual_size', 'pVSz'),
 ('data', 'pPCT'),
 ('description', 'pDes'),
 ('downloaded', 'pDlA'),
 ('format', 'pFmt'),
 ('kind', 'pKnd'),
 ('raw_data', 'pRaw'),
 ('artist', 'pArt'),
 ('compilation', 'pAnt'),
 ('composer', 'pCmp'),
 ('disc_count', 'pDsC'),
 ('disc_number', 'pDsN'),
 ('genre', 'pGen'),
 ('year', 'pYr '),
 ('location', 'pLoc'),
 ('minimized', 'pMin'),
 ('view', 'pPly'),
 ('band_1', 'pEQ1'),
 ('band_2', 'pEQ2'),
 ('band_3', 'pEQ3'),
 ('band_4', 'pEQ4'),
 ('band_5', 'pEQ5'),
 ('band_6', 'pEQ6'),
 ('band_7', 'pEQ7'),
 ('band_8', 'pEQ8'),
 ('band_9', 'pEQ9'),
 ('band_10', 'pEQ0'),
 ('modifiable', 'pMod'),
 ('preamp', 'pEQA'),
 ('update_tracks', 'pUTC'),
 ('container', 'ctnr'),
 ('id', 'ID  '),
 ('index', 'pidx'),
 ('persistent_ID', 'pPIS'),
 ('duration', 'pDur'),
 ('parent', 'pPlP'),
 ('shuffle', 'pShf'),
 ('size', 'pSiz'),
 ('song_repeat', 'pRpt'),
 ('special_kind', 'pSpK'),
 ('time', 'pTim'),
 ('visible', 'pvis'),
 ('capacity', 'capa'),
 ('free_space', 'frsp'),
 ('album', 'pAlb'),
 ('album_artist', 'pAlA'),
 ('album_rating', 'pAlR'),
 ('album_rating_kind', 'pARk'),
 ('bit_rate', 'pBRt'),
 ('bookmark', 'pBkt'),
 ('bookmarkable', 'pBkm'),
 ('bpm', 'pBPM'),
 ('category', 'pCat'),
 ('comment', 'pCmt'),
 ('database_ID', 'pDID'),
 ('date_added', 'pAdd'),
 ('enabled', 'enbl'),
 ('episode_ID', 'pEpD'),
 ('episode_number', 'pEpN'),
 ('EQ', 'pEQp'),
 ('finish', 'pStp'),
 ('gapless', 'pGpl'),
 ('grouping', 'pGrp'),
 ('long_description', 'pLds'),
 ('lyrics', 'pLyr'),
 ('modification_date', 'asmo'),
 ('played_count', 'pPlC'),
 ('played_date', 'pPlD'),
 ('podcast', 'pTPc'),
 ('rating', 'pRte'),
 ('rating_kind', 'pRtk'),
 ('release_date', 'pRlD'),
 ('sample_rate', 'pSRt'),
 ('season_number', 'pSeN'),
 ('shufflable', 'pSfa'),
 ('skipped_count', 'pSkC'),
 ('skipped_date', 'pSkD'),
 ('show', 'pShw'),
 ('sort_album', 'pSAl'),
 ('sort_artist', 'pSAr'),
 ('sort_album_artist', 'pSAA'),
 ('sort_name', 'pSNm'),
 ('sort_composer', 'pSCm'),
 ('sort_show', 'pSSN'),
 ('start', 'pStr'),
 ('track_count', 'pTrC'),
 ('track_number', 'pTrN'),
 ('unplayed', 'pUnp'),
 ('video_kind', 'pVdK'),
 ('volume_adjustment', 'pAdj'),
 ('address', 'pURL'),
 ('shared', 'pShr'),
 ('smart', 'pSmt'),
 ('bounds', 'pbnd'),
 ('closeable', 'hclb'),
 ('collapseable', 'pWSh'),
 ('collapsed', 'wshd'),
 ('position', 'ppos'),
 ('resizable', 'prsz'),
 ('zoomable', 'iszm'),
 ('zoomed', 'pzum')]

elements = \
[('artworks', 'cArt'),
 ('audio_CD_playlists', 'cCDP'),
 ('audio_CD_tracks', 'cCDT'),
 ('browser_windows', 'cBrW'),
 ('device_playlists', 'cDvP'),
 ('device_tracks', 'cDvT'),
 ('encoders', 'cEnc'),
 ('EQ_presets', 'cEQP'),
 ('EQ_windows', 'cEQW'),
 ('file_tracks', 'cFlT'),
 ('folder_playlists', 'cFoP'),
 ('items', 'cobj'),
 ('library_playlists', 'cLiP'),
 ('playlists', 'cPly'),
 ('playlist_windows', 'cPlW'),
 ('radio_tuner_playlists', 'cRTP'),
 ('shared_tracks', 'cShT'),
 ('sources', 'cSrc'),
 ('tracks', 'cTrk'),
 ('URL_tracks', 'cURT'),
 ('user_playlists', 'cUsP'),
 ('visuals', 'cVis'),
 ('windows', 'cwin'),
 ('application', 'capp'),
 ('print_settings', 'pset')]

commands = \
[('set', 'coresetd', [('to', 'data')]),
 ('exists', 'coredoex', []),
 ('move', 'coremove', [('to', 'insh')]),
 ('subscribe', 'hookpSub', []),
 ('playpause', 'hookPlPs', []),
 ('download', 'hookDwnl', []),
 ('close', 'coreclos', []),
 ('open', 'aevtodoc', []),
 ('open_location', 'GURLGURL', []),
 ('quit', 'aevtquit', []),
 ('pause', 'hookPaus', []),
 ('make',
  'corecrel',
  [('new', 'kocl'), ('at', 'insh'), ('with_properties', 'prdt')]),
 ('duplicate', 'coreclon', [('to', 'insh')]),
 ('print_',
  'aevtpdoc',
  [('print_dialog', 'pdlg'),
   ('with_properties', 'prdt'),
   ('kind', 'pKnd'),
   ('theme', 'pThm')]),
 ('add', 'hookAdd ', [('to', 'insh')]),
 ('rewind', 'hookRwnd', []),
 ('play', 'hookPlay', [('once', 'POne')]),
 ('run', 'aevtoapp', []),
 ('resume', 'hookResu', []),
 ('updatePodcast', 'hookUpd1', []),
 ('next_track', 'hookNext', []),
 ('stop', 'hookStop', []),
 ('search', 'hookSrch', [('for_', 'pTrm'), ('only', 'pAre')]),
 ('updateAllPodcasts', 'hookUpdp', []),
 ('update', 'hookUpdt', []),
 ('previous_track', 'hookPrev', []),
 ('fast_forward', 'hookFast', []),
 ('count', 'corecnte', [('each', 'kocl')]),
 ('reveal', 'hookRevl', []),
 ('convert', 'hookConv', []),
 ('eject', 'hookEjct', []),
 ('back_track', 'hookBack', []),
 ('refresh', 'hookRfrs', []),
 ('delete', 'coredelo', [])]



#!/usr/bin/env  python
__license__   = 'GPL v3'
__copyright__ = '2008, Kovid Goyal kovid@kovidgoyal.net'
__docformat__ = 'restructuredtext en'

from PyQt5.Qt import QDialog, Qt, QPixmap, QIcon

from calibre import confirm_config_name
from calibre.gui2 import dynamic
from calibre.gui2.dialogs.confirm_delete_ui import Ui_Dialog

class Dialog(QDialog, Ui_Dialog):

    def __init__(self, msg, name, parent, config_set=dynamic):
        QDialog.__init__(self, parent)
        self.setupUi(self)

        self.msg.setText(msg)
        self.name = name
        self.again.stateChanged.connect(self.toggle)
        self.buttonBox.setFocus(Qt.OtherFocusReason)
        self.config_set = config_set

    def toggle(self, *args):
        self.config_set[confirm_config_name(self.name)] = self.again.isChecked()


def confirm(msg, name, parent=None, pixmap='dialog_warning.png', title=None,
        show_cancel_button=True, confirm_msg=None, config_set=None):
    config_set = config_set or dynamic
    if not config_set.get(confirm_config_name(name), True):
        return True
    d = Dialog(msg, name, parent, config_set=config_set)
    d.label.setPixmap(QPixmap(I(pixmap)))
    d.setWindowIcon(QIcon(I(pixmap)))
    if title is not None:
        d.setWindowTitle(title)
    if not show_cancel_button:
        d.buttonBox.button(d.buttonBox.Cancel).setVisible(False)
    if confirm_msg is not None:
        d.again.setText(confirm_msg)
    d.resize(d.sizeHint())
    return d.exec_() == d.Accepted
